﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Excel = Microsoft.Office.Interop.Excel;
using Office = Microsoft.Office.Core;
using Microsoft.Office.Tools.Excel;
using Microsoft.Office.Tools;

namespace HBaseExcelAddIn
{
    public partial class HBasePane : UserControl
    {
        private string[] columns;
        private string filterColumn;
        private string filterValue;
        private string frequencyColumn;
        private Excel.Range cols;

        public HBasePane()
        {
            InitializeComponent();
        }

        private void viewTableButtonClick(object sender, EventArgs e)
        {
            if (String.IsNullOrEmpty(this.hostName.Text))
            {
                MessageBox.Show("Zookeeper host name is empty", "Invalid configuration", MessageBoxButtons.OK);
                return;
            }

            if (String.IsNullOrEmpty(this.tableName.Text))
            {
                MessageBox.Show("Table name is empty", "Invalid configuration", MessageBoxButtons.OK);
                return;
            }

            Excel.Worksheet activeWorksheet = ((Excel.Worksheet)Globals.ExcelHBaseAddIn.Application.ActiveSheet);
            activeWorksheet.Name = "Records";
            Excel.Range navigator = activeWorksheet.get_Range("A1");

            int numRows = Decimal.ToInt32(this.numberOfRecords.Value);
            // most of the work done here
            this.columns = ViewHBaseTable.populateWorkSheet(navigator
                , this.hostName.Text
                , this.tableName.Text
                , numRows);
            // autofit the range
            int numCols = this.columns.Count<string>();
            Excel.Range c1 = activeWorksheet.Cells[1, 1];
            Excel.Range c2 = activeWorksheet.Cells[numRows, numCols];
            this.cols = activeWorksheet.get_Range(c1, c2); 
            this.cols.EntireColumn.AutoFit();
            // populate the user controls with the column names
            this.filterComboBox.Items.AddRange(this.columns);
            this.frequencyComboBox.Items.AddRange(this.columns);
        }

        private void onSelectionCommited(object sender, EventArgs e)
        {
            this.filterColumn =  this.filterComboBox.SelectedText;
            this.frequencyComboBox.Items.Clear();
            this.frequencyComboBox.Items.AddRange(this.columns);
            this.frequencyComboBox.Items.Remove(this.filterColumn);     
        }

        private void onMapRedButtonClick(object sender, EventArgs e)
        {
            this.filterColumn = this.filterComboBox.Text;
            if (String.IsNullOrEmpty(this.filterValueTextBox.Text))
            {
                MessageBox.Show("Filter value is empty", "Invalid configuration", MessageBoxButtons.OK);
                return;
            }
            this.filterValue = this.filterValueTextBox.Text;
            this.frequencyColumn = this.frequencyComboBox.Text;

            Excel.Worksheet activeWorksheet = ((Excel.Worksheet)Globals.ExcelHBaseAddIn.Application.Worksheets[2]);
            activeWorksheet.Name = "Frequency";
            Excel.Range navigator = activeWorksheet.get_Range("A1");
            // most of the fun stuff happens here
            int numRows = MapReduce.executeMapReduce(navigator
                , this.filterColumn
                , this.filterValue
                , this.frequencyColumn
                , this.hostName.Text
                , this.tableName.Text);
            // autofit the range
            Excel.Range c1 = activeWorksheet.Cells[1, 1];
            Excel.Range c2 = activeWorksheet.Cells[numRows, 2];
            this.cols = activeWorksheet.get_Range(c1, c2); 
            this.cols.EntireColumn.AutoFit();
            // bring the worksheet to the top
            activeWorksheet.Activate();
        }

        private void onChartFrequenciesClick(object sender, EventArgs e)
        {
            Excel.Workbook wb = Globals.ExcelHBaseAddIn.Application.ActiveWorkbook;
            Excel.Chart chart = (Excel.Chart)wb.Charts.Add();
            chart.ChartType = Excel.XlChartType.xlColumnStacked;
            chart.SetSourceData(this.cols, Excel.XlRowCol.xlColumns);
            chart.HasTitle = true;
            string filterName = this.filterColumn.Substring(this.filterColumn.IndexOf(":") + 1);
            string frequencyName = this.frequencyColumn.Substring(this.frequencyColumn.IndexOf(":") + 1);
            chart.ChartTitle.Text = "Frequency of " + frequencyName + " when " + filterName + " = " + this.filterValue;
        }
    }
}
