//-----------------------------------------------------------------------------------------------------------
//
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
// KIND, WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
// PURPOSE.
//
//------------------------------------------------------------------------------------------------------------
package HBaseToLinq;

import java.io.IOException;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.client.Put;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.client.Scan;
import org.apache.hadoop.hbase.filter.FirstKeyOnlyFilter;
import org.apache.hadoop.hbase.filter.SingleColumnValueFilter;
import org.apache.hadoop.hbase.filter.CompareFilter;
import org.apache.hadoop.hbase.io.ImmutableBytesWritable;
import org.apache.hadoop.hbase.mapreduce.TableMapReduceUtil;
import org.apache.hadoop.hbase.mapreduce.TableMapper;
import org.apache.hadoop.hbase.mapreduce.TableReducer;
import org.apache.hadoop.hbase.util.Bytes;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Job;

public class FrequencyMapRed {
	
	private static  byte[] family;
    private static  byte[] cellToCount;
    //private static  byte[] pageCol = Bytes.toBytes("page");
    
    private static String getFamily(String row)
    {
    	return row.substring(0, row.indexOf(":"));
    }
    
    private static String getCell(String row)
    {
    	return row.substring(row.indexOf(":") + 1);
    }
    
    private static byte[] getFamilyAsBytes(String row)
    {
    	return Bytes.toBytes(getFamily(row));
    }
    
    private static byte[] getCellAsBytes(String row)
    {
    	return Bytes.toBytes(getCell(row));
    }
    
    static class Mapper1 extends TableMapper<Text, IntWritable> {

        private static final IntWritable one = new IntWritable(1);
        private Text txt = new Text();  
        @Override
        public void map(ImmutableBytesWritable row, Result values, Context context) throws IOException {
            // extract userKey from the compositeKey (userId + counter)      
        	String val = new String(values.getValue(family, cellToCount));
            txt.set(val);
            try {
                context.write(txt, one);
            } catch (InterruptedException e) {
                throw new IOException(e);
            }
        }
    }

    public static class Reducer1 extends TableReducer<Text, IntWritable, ImmutableBytesWritable> {

        public void reduce(Text key, Iterable<IntWritable> values, Context context)
                throws IOException, InterruptedException {
            int sum = 0;
            for (IntWritable val : values) {
                sum += val.get();
            }

            Put put = new Put(Bytes.toBytes(key.toString()));
            put.add(family, Bytes.toBytes("total"), Bytes.toBytes(sum));
            //System.out.println(String.format("stats :   key : %d,  count : %d", Bytes.toInt(key.get()), sum));
            context.write(null, put);
        }
    }

    
    public static void executeMapRed(String hostName, String tableName, String columnToCount, String filterColumn, String filterColumnValue) throws Exception {
        Configuration conf = HBaseConfiguration.create();
        conf.set("hbase.zookeeper.quorum", hostName /*"localhost.localdomain"*/);
        Job job = null;
        try
        {
        	family = getFamilyAsBytes(columnToCount);
        	cellToCount = getCellAsBytes(columnToCount);
	        job = new Job(conf, "Hbase_FreqCounter1");
	        job.setJarByClass(FrequencyMapRed.class);
	        Scan scan = new Scan();
	        //scan.addColumn(family, ipCol);
	        scan.setCaching(500);        
	        scan.setCacheBlocks(false);
	        SingleColumnValueFilter svcFilter = new SingleColumnValueFilter(getFamilyAsBytes(filterColumn)
	                , getCellAsBytes(filterColumn)
	                , CompareFilter.CompareOp.EQUAL
	                , Bytes.toBytes(filterColumnValue));
	        
	        
	        scan.setFilter(svcFilter);
	        TableMapReduceUtil.initTableMapperJob(tableName /*"access_logs"*/, scan, Mapper1.class, Text.class,
	                IntWritable.class, job);
	        TableMapReduceUtil.initTableReducerJob("summary_user", Reducer1.class, job);
	        job.waitForCompletion(true);
        }
        catch (Exception ex)
	    {
        	throw ex;
	    }  
    }

}
